# SUPPLEMENTARY MATERIAL: bowman_et_al_code_for_test_dataset
#===========================================================================================================================#
## NOTES:
# Test dataset from McLaughlin et al. & Bennett et al 
# Importing 'Protein_profiling_test_set_VB_JM.xlsx' and cleaning steps 
# 16 Unknown samples labelled 'UN1' to 'UN16'
# Detecting electrophoretic protein peaks using PROcess and 'isPeak'
# 12 informative time windows
# Classified using Boolean Logic Paths with up to 12 active gates.
# There are 7 paths, as Dipel and B. thuringiensis are treated as one sample.
#===========================================================================================================================#
######################################################################
# IMPORTING DATA FROM EXCEL WORKBOOK #
######################################################################
# install.packages("openxlsx",dependencies=TRUE)
library(openxlsx) #to read and write excel spreadsheets ".xlsx"

# DATA contains the first column as time points (seconds), then each sample and its replicate in proceeding columns.
# Each individual electropherogram is IN columns NOT the row.
# DATA containing unknown headings (sheet 1)
data.TEST <- read.xlsx("Supplementary File 3.xlsx",sheet=1)

#---------------------------------------------------------------------------------------------------------------------------#
######################################################################
# DATA CLEANING STEP 1 #
######################################################################
# ALL negative values of fluorescence (y-axis) from 0-70 seconds were replaced by zero:
data.TEST[which(data.TEST <0,arr.ind=TRUE)]<-0

#----------------------------------------------------------#
# Graphical representations of output from cleaning step 1 #
#----------------------------------------------------------#
# Calculate maximum and minimum limits
maximum<-apply(data.TEST,2,max)
el.max<-max(maximum)
minimum<-apply(data.TEST,2,min)
el.min<-min(minimum)
# The minimum is already zero

#------------------------------------------#
pdf("graphics_clean_up_step_1_unknown.pdf")
#------------------------------------------#
plot(data.TEST[,2],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 1")
plot(data.TEST[,3],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 2")
plot(data.TEST[,4],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 3")
plot(data.TEST[,5],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 4")
plot(data.TEST[,6],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 5")
plot(data.TEST[,7],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 6")
plot(data.TEST[,8],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 7")
plot(data.TEST[,9],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 8")
plot(data.TEST[,10],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 9")
plot(data.TEST[,11],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 10")
plot(data.TEST[,12],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 11")
plot(data.TEST[,13],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 12")
plot(data.TEST[,14],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 13")
plot(data.TEST[,15],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 14")
plot(data.TEST[,16],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 15")
plot(data.TEST[,17],type="l",col="red",ylim=c(el.min, el.max), main="Unknown 16")
#--------#
dev.off()
#--------#

#---------------------------------------------------------------------------------------------------------------------------#
######################################################################
# DATA CLEANING STEP 2 #
######################################################################
# Truncated from 22.45 sec - 62.45 sec (rows 450-1250).
# This data is between the system peak of the intstrument and the upper internal marker.
data.TEST <- data.TEST[450:1250,]

#----------------------------------------------------------#
# Graphical representations of output from cleaning step 2 #
#----------------------------------------------------------#
# Calculate maximum and minimum limits
maximum<-apply(data.TEST,2,max)
el.max<-max(maximum)
minimum<-apply(data.TEST,2,min)
el.min<-min(minimum)
# The minimum is already zero

#------------------------------------------#
pdf("graphics_clean_up_step_2_unknown.pdf")
#------------------------------------------#
plot(data.TEST[,2],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 1")
plot(data.TEST[,3],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 2")
plot(data.TEST[,4],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 3")
plot(data.TEST[,5],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 4")
plot(data.TEST[,6],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 5")
plot(data.TEST[,7],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 6")
plot(data.TEST[,8],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 7")
plot(data.TEST[,9],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 8")
plot(data.TEST[,10],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 9")
plot(data.TEST[,11],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 10")
plot(data.TEST[,12],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 11")
plot(data.TEST[,13],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 12")
plot(data.TEST[,14],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 13")
plot(data.TEST[,15],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 14")
plot(data.TEST[,16],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 15")
plot(data.TEST[,17],type="l",col="blue",ylim=c(el.min, el.max), main="Unknown 16")
#--------#
dev.off()
#--------#

#---------------------------------------------------------------------------------------------------------------------------#
######################################################################
# LOCATE PEAKS USING PROCESS AND OPTIMIZED PARAMETERS #
######################################################################
# "PROcess": Is a package used for electropherogram analysis to identify protein peaks. 

# To install the package "PROcess":
# http:// if https:// URLs are not supported
# source("https://bioconductor.org/biocLite.R")
# biocLite("PROcess")

library(PROcess)

## 'isPeak': to locate peaks in a spectra using PROcess package

# USED ARGUMENTS:
## 'SoN': signal to noise ratio for peak detection
## 'span': parameter for estimating local variance before peak detection
## 'sm.span': parameter for smoothing electropherogram before peak detection

## TO LOCATE, IDENTIFY & DETECT (THREE MAJOR STEPS):
#1# Smoothing the electrpherogram uses the moving averages of K-nearest neighbours and controlled by the 'sm.span' parameter/argument.
#2# Compute local variability as the median of the absolute deviations of K-nearest neighbours and 'span' argument.
#3# Identify local maxima of the smoothed spectrum using three methods:
###A# The SNR (signal to noise ratio, which is the 'SoN' argument): local smooth/local variability. 
###B# The intensty threshold for the whole spectrum ('zerothrsh' argument).
###C# The shape ratio: the area under the curve within a small distance of a peak candidate/maximum of all such peaks.

#---------------------------------#
# Functions: PEAKS and AREAS #
#---------------------------------#

# THESE ARE THE OPTIMAL PARAMETERS FOR FUNCTIONS TO FIND THE PEAKS AND AREAS

time<-data.TEST[,1]; # These are the time points
data.TEST <- data.TEST[,-1]; # These REMOVE the time points

# NOTE: 'span' and 'sm.span' must have a value that is uneven

PEAKS<-function(y){isPeak(cbind(time,y),SoN=2, span=25, sm.span=11, plot=TRUE, area.w =0.01, ratio = 0.3, zerothrsh=5)$peak}

AREAS<-function(y){isPeak(cbind(time,y),SoN=2, span=25, sm.span=11, area.w =0.01, ratio = 0.3, zerothrsh=5)$area}

#---------------------------------------------------------------------------------------------------------------------------#
#############################
# TEST SET - IDENTIFY PEAKS #
#############################

# To identify peaks in unknown samples according to optimal parameters in PEAKS function
apply(data.TEST,2,PEAKS)->peaksUK

# To determine the number of peaks in each unknown sample
nPeaks <- apply(peaksUK,2,sum)

# To determine location of peaks (according to row number that can than be matched to time points)
# Format as a list
apply(peaksUK,2,function(x){which(x==TRUE)}) -> peakUKsites 

# We used a list, as not all samples have the same number or location/site of peaks
# Re-format as a matrix 

#---------------------------------------------------------------------------#
# Convert the list of 'PEAKS' to a matrix with detected peaks and locations #
#---------------------------------------------------------------------------#

#install.packages("reshape2")
library(reshape2)
melt(peakUKsites) -> peakUKsites.mat

#-----------------------------------------------------------------------------------#
# Graphical representation of individual samples and the location of peaks detected #
#-----------------------------------------------------------------------------------#
#------------------#
pdf("PeaksUK.pdf")
#------------------#
for( i in 1:16){
  isPeak(cbind(time, data.TEST[,i]),SoN=2, span=25, sm.span=11, plot=TRUE, area.w =0.01, ratio = 0.3, zerothrsh=5)	
}
#--------#
dev.off()
#--------#

#---------------------------------------------#
# Calculate the value of the area for samples #
#---------------------------------------------#
# If not there is no peak, use of NA
apply(data.TEST,2,AREAS)->AreasUK

#---------------------------------------------------------------------------------------------------------------------------#
#########################################################################################################
# Split character data into numbers and letters (to use only the number associated with each replicate) #
#########################################################################################################

#install.packages("stringr")
library(stringr)

# Modifying peakUKsites.mat format
colnames(peak2UKsites.mat)<-c("PeakSite","Replica")
peak2UKsites.mat
peak2UKsites.mat$Replica.Num<-as.numeric(str_extract(peak2UKsites.mat$Replica, "[0-9]+"))

#---------------------------------------------------------------------------------------------------------------------------#

############################
# 12 Specific time windows #  
############################
## Peaks for each sample are in a matrix format as 'peakUKsites.mat' with columns= [,1] Peak site/location, [,2] Sample name (UN1 to UN16) & [,3] Replicate number (1-16). 
matrix.peaks<- peakUKsites.mat
dim(matrix.peaks)

# New MATRIX named 'peaks' 
# Has 16 Unknowns / 12 specific time windows 
peaks<- matrix(nrow=16,ncol=12)

## From matrix.peaks we specify, using 2 vectors consisting of our extreme.left and extreme.right sides of a time window.
## For example, the time window of 32.40-34.65 seconds, is from rows 200 (extreme.left or the smallest) to 245 (extreme.right or the largest) make up that target site.
## Time windows do not extend beyond 46.00 seconds.

extreme.left.peaks <- c(200,360,275,140,55,30,110,130,180,100,170,149)
extreme.right.peaks <- c(245,460,335,175,95,55,145,180,230,180,212,164)

## LOOP is used to search the matrix.peaks for each peak in a specified window (by extreme.left and extreme.right) and return the number of peaks found in that window. 
for(i in 1:12){
  table(matrix.peaks[,1]>extreme.left.peaks[i] & matrix.peaks[,1]<extreme.right.peaks[i], matrix.peaks[,2])[2,] -> peaks[,i]
}

# Ending with the PEAK MATRIX ('peaks') with the number of peaks detected in the 12 specified time windows (columns= 12), for all sample unknowns (rows= 16) 
peaks
rownames(peaks)<-names(table(matrix.peaks[,2]))

# NOTE:
# ROWS '200-245' is equal to the 32.40-34.65 second time window, also refered to as 'PEAK1' or 'peaks[,1]'  
# ROWS '360-460' is equal to the 40.40-45.40 second time window, also refered to as 'PEAK2' or 'peaks[,2]'
# ROWS '275-335' is equal to the 36.15-39.15 second time window, also refered to as 'PEAK3' or 'peaks[,3]'   
# ROWS '140-175' is equal to the 29.65-31.15 second time window, also refered to as 'PEAK4' or 'peaks[,4]'   
# ROWS '55-95' is equal to the 25.15-27.15 second time window, also refered to as 'PEAK5' or 'peaks[,5]'   
# ROWS '30-55' is equal to the 23.90-25.15 second time window, also refered to as 'PEAK6' or 'peaks[,6]'   
# ROWS '110-145' is equal to the 27.90-29.60 second time window, also refered to as 'PEAK7' or 'peaks[,7]'  
# ROWS '130-180' is equal to the 28.90-31.40 second time window, also refered to as 'PEAK8' or 'peaks[,8]'   
# ROWS '180-230' is equal to the 31.40-33.90 second time window, also refered to as 'PEAK9' or 'peaks[,9]' 
# ROWS '100-180' is equal to the 27.80-31.40 second time window, also refered to as 'PEAK10' or 'peaks[,10]'   
# ROWS '170-212' is equal to the 30.90-33.0 second time window, also refered to as 'PEAK11' or 'peaks[,11]'  
# ROWS '149-164' is equal to the 29.85-30.60 second time window, also refered to as 'PEAK12' or 'peaks[,12]' 

#---------------------------------------------------------------------------------------------------------------------------#
################################################################
# CLASSIFICATION FOR 7 SAMPLES USING BOOLEAN LOGIC PATHS #
################################################################
# SAMPLES have 7 columns as Dipel and B. thuringiensis are treated as one classification.
# 16 ROWS as there unknowns

# NOTE:
# "D/BT" = powder-based ('dipel') and culture-based B. thuringiensis Kurstaki strain
# "BA" = B. anthracis Sterne strain
# "BC" = B. cereus
# "FL" = plain wheat flour
# "YE" = Y. enterocolitica B1A S09
# "YP" = Y. pseudotuberculosis
# "EC" = E. coli

SAMPLES <- matrix(nrow=16,ncol=7)
colnames(SAMPLES) <- c("D/BT","BA","BC","FL","YE","YP","EC")
rownames(SAMPLES)

## Boolean logic paths use up to 12 active gates with specific time windows (columns) and threshold of peaks (present or absent) as parameters.
## The Logic values of TRUE/FALSE are stored in the 'SAMPLES' matrix.
## Boolean logic are built using operators OR (|) and AND (&) here.

# Use of equals '==', less than or equal to '<=', greater than or equal to '>='
# Use of 'apply', is calculated by 12 (time windows) minus the total indicative peaks (represented by equals '==' e.g. ==1 or ==2)

#-------------------------------------------------------------------------------------------------------#
# Path to classify "powder-based ('dipel') and culture-based B.thuringiensis Kurstaki strain" or "D/BT" #
#-------------------------------------------------------------------------------------------------------#
SAMPLES[,1]<-peaks[,1]<=1 & (peaks[,2]==1 | peaks[,2]==2) & peaks[,3]==0 & peaks[,5]==0 & 
  peaks[,9]<=1 & apply(peaks,1,sum)<=10
peaks[SAMPLES[,1]==1,]

#-------------------------------------------------------------------------------------------------------#
# Path to classify "B. anthracis Sterne strain" or "BA" #
#-------------------------------------------------------------------------------------------------------#
SAMPLES[,2]<- peaks[,1]==0 & peaks[,2]==0 & peaks[,3]==1 & peaks[,6]==0 & peaks[,9]==0 & 
  peaks[,10]<=1 & peaks[,11]==0 & apply(peaks,1,sum)<=11
peaks[SAMPLES[,2]==1,]

#-------------------------------------------------------------------------------------------------------#
# Path to classify "B. cereus" or "BC" #
#-------------------------------------------------------------------------------------------------------#
SAMPLES[,3]<-  peaks[,1]==0 & peaks[,2]==0 & peaks[,3]==0 & peaks[,4]>=1 & 
  (peaks[,6]==1 | peaks[,7]==1) & peaks[,5]==1 & peaks[,8]>=1 & peaks[,9]==0 & peaks[,10]>=1 &
  apply(peaks,1,sum)<=10
peaks[SAMPLES[,3]==1,]

#-------------------------------------------------------------------------------------------------------#
# Path to classify "plain wheat flour" or "FL" #
#-------------------------------------------------------------------------------------------------------#
SAMPLES[,4]<- peaks[,2]==0 &  peaks[,3]==0 & peaks[,5]==0 & peaks[,6]==0 & peaks[,7]==0 &
  (peaks[,8]==1 | peaks[,11]==1) & peaks[,9]==1 & peaks[,10]<=1  &  apply(peaks,1,sum)<=10
peaks[SAMPLES[,4]==1,]

#-------------------------------------------------------------------------------------------------------#
# Path to classify "Y. enterocolitica B1A S09" or "YE" #
#-------------------------------------------------------------------------------------------------------#
SAMPLES[,5]<-peaks[,1]==0 & peaks[,2]==0 & peaks[,3]==0 & peaks[,4]==1 & peaks[,5]==0 & 
  peaks[,6]==0 & peaks[,7]==0 & peaks[,8]==1 & peaks[,9]==0 & peaks[,10]==1  & 
  peaks[,12]==1 & apply(peaks,1,sum)<=8
peaks[SAMPLES[,5]==1,]

#-------------------------------------------------------------------------------------------------------#
# Path to classify "Y. pseudotuberculosis" or "YP" #
#-------------------------------------------------------------------------------------------------------#
SAMPLES[,6]<- peaks[,2]==0 & peaks[,3]==0 & peaks[,5]<=1 & peaks[,6]>=1 & 
  peaks[,7]==1 & peaks[,8]<=1 & peaks[,9]<=1 & peaks[,10]>=1 & peaks[,11]==1 & 
  apply(peaks,1,sum)<= 10
peaks[SAMPLES[,6]==1,]

#-------------------------------------------------------------------------------------------------------#
# Path to classify "E. coli" or "EC" #
#-------------------------------------------------------------------------------------------------------#
SAMPLES[,7]<- peaks[,2]==0 & peaks[,3]==0 & peaks[,4]==1 & (peaks[,5]==2 | peaks[,10]==2) &
  (peaks[,6]==0 | peaks[,7]==0)  & peaks[,8]>=1 & apply(peaks,1,sum)<=11  
peaks[SAMPLES[,7]==1,]

#---------------------------------------------------------------------------------------------------------------------------#
# Table of replicates with BINARY TRUE/FALSE classification for each unknown:
rownames(SAMPLES)<-names(table(matrix.peaks[,2]))
SAMPLES

# Number of samples classified in each Boolean logic path:
CLASS <- apply(SAMPLES,2,sum)
CLASS

# Add column names = 'PEAKS' to the Peak matrix
PEAKS1 <- data.frame(peaks)
colnames(PEAKS1)<-c("PEAK1","PEAK2","PEAK3","PEAK4","PEAK5","PEAK6","PEAK7","PEAK8","PEAK9","PEAK10", "PEAK11", "PEAK12")

# For peak matrix export
#library(xlsx)
#write.xlsx(PEAKS1, file="peak_matrix.xlsx", sheetName="test_PEAK_matrix", row.names = TRUE)

#--------------------------------------------------------------------------------------------------------------------------#




